import clr
clr.AddReference('IronPython.Wpf')
import wpf
from System.Windows.Controls import StackPanel
from System.IO import StreamReader
from System.Windows.Input import Keyboard
from System import Type, Tuple, Array, Double
from System.Windows.Threading import Dispatcher
clr.AddReference("System.Drawing")
from System.Drawing import Bitmap
clr.AddReference("System.Windows.Forms")
from System.Windows.Forms import Control, Keys
clr.AddReference("Trimble.DiskIO")
from Trimble.DiskIO import SearchPath, OptionsManager
clr.AddReference("Trimble.Vce.Core")
from Trimble.Vce.Core import TransactMethodCall, ModelEvents
from Trimble.Vce.Core.Components import WorldView, Project, Layer, TextStyle
clr.AddReference("Trimble.Vce.Alignment")
from Trimble.Vce.Alignment.Linestring import Linestring,VerticalElementFactory, IVerticalNoCurve, IStationVerticalLocation
from Trimble.Vce.Alignment.Linestring import ElementFactory, IStraightSegment, IXYZLocation,ITangentArcSegment, ArcTangentType
clr.AddReference("Trimble.Vce.Geometry")
from Trimble.Vce.Geometry import Point3D, Intersections
from Trimble.Vce.Geometry.PolySeg import PolySeg
clr.AddReference("Trimble.Vce.Interfaces")
from Trimble.Vce.Interfaces import Client
clr.AddReference("Trimble.Vce.UI.Controls")
#from Trimble.Vce.UI.Controls import MousePosition, CursorStyle, GlobalSelection
# later version of TBC moved these to a different assembly. If not found in new location, look at old
try:
    clr.AddReference("Trimble.Vce.ViewModel")
    clr.AddReference("Trimble.Vce.Units")
    from Trimble.CustomControl.Interfaces import MousePosition
    from Trimble.CustomControl.Interfaces.Enums import CursorStyle
except:
    from Trimble.Vce.UI.Controls import MousePosition, CursorStyle
from Trimble.Vce.UI.Controls import GlobalSelection

from Trimble.Vce.UI.Controls.Wpf import CoordinateEdit, DistanceEdit
from Trimble.CustomControl.Interfaces.Enums import InputMethod

def Setup(cmdData, macroFileFolder):
    cmdData.Key = "InsertLineSegment"
    cmdData.CommandName = "InsertLineSegment"
    cmdData.Caption = "_Insert Line Segment"
    cmdData.UIForm = "InsertLineSegment"
    cmdData.HelpFile = "Macros.chm"
    cmdData.HelpTopic = "22996"

    cmdData.DefaultTabKey = "Macros"
    cmdData.DefaultTabGroupKey = "CAD"
    cmdData.ShortCaption = "Insert Line Segment"
    cmdData.ToolTipTittle = "Insert Line Segment"
    cmdData.ToolTipTextFormatted = "Inserts a straight line segment at defined location."
    try:
        b = Bitmap(macroFileFolder + "\\" + cmdData.Key + ".png")
        cmdData.ImageSmall = b
    except:
        pass

class InsertLineSegment(StackPanel): # this inherits from the WPF StackPanel control
    def __init__(self, currentProject, macroFileFolder):
        with StreamReader(macroFileFolder + r"\InsertLineSegment.xaml") as s:
            wpf.LoadComponent(self, s)
        self.currentProject = currentProject

    def OnLoad(self, cmd, buttons, event):
        buttons[0].Content = "Apply"
        buttons[1].Content = "Close"
        self.Caption = cmd.Command.Caption
        self.lineCtl.AddTypeFilter(clr.GetClrType(Linestring))
        OptionsManager.SetValue("InsertLine:Elev", self.elevCtl.AutoTabExtSkip)

    def lineCtl_ValueChanged(self, ctrl, e):
        if (self.lineCtl.SerialNumber):
            selectionSet = GlobalSelection.Items(self.currentProject)
            selectionSet.Set([self.lineCtl.SerialNumber])

    def coordCtl_ValueChanged(self, ctrl, e):
        # run cmd if change was due to mouse pick
        if e.Cause == InputMethod.Mouse:
            if self.elevCtl.AutoTabExtSkip:       
                self.OkClicked(None, None)
            else:
                self.elevCtl.Elevation = self.coordCtl.Coordinate.Z
                Keyboard.Focus(self.elevCtl)

    def elevCtl_ValueChanged(self, ctrl, e):
        if e.Cause == InputMethod.Mouse:     
            self.OkClicked(None, None)

    def CancelClicked(self, thisCmd, args):
        thisCmd.CloseUICommand()

    def OkClicked(self, thisCmd, e):
        line = self.lineCtl.Entity
        if not line:
            self.lineCtl.StatusMessage = "No line selected"
            return
        self.lineCtl.StatusMessage = ""
        coord = self.coordCtl.Coordinate
        if not coord.Is3D and not coord.Is2D :
            self.coordCtl.StatusMessage = "No coordinate defined"
            return
        self.coordCtl.StatusMessage = ""
        e = self.elevCtl.Elevation
        if not Double.IsNaN(e):
            coord.Z = e
            self.elevCtl.Clear() # clear elevation after use

        # tell undo manager we are starting to do something
        self.currentProject.TransactionManager.AddBeginMark(Client.CommandGranularity.Command, self.Caption)

        try:
            with TransactMethodCall(self.currentProject.TransactionCollector) as failGuard:
                poly = line.ComputePolySeg()
                if not poly or poly.NumberOfNodes < 2:
                    self.lineCtl.StatusMessage = "No line selected"
                    return
                rtn = poly.FindPointFromPoint(coord, None, None)
                if not rtn[0]:
                    return
                seg = rtn[2]
                if not seg:
                    return
                index = seg.Node.Index
                e = ElementFactory.Create(clr.GetClrType(IStraightSegment), clr.GetClrType(IXYZLocation))
                e.Position = coord
                line.InsertElementAt(e, index)

                # you MUST commit the change to DB or they will all be un-done
                # (roll-back)
                failGuard.Commit()
        finally:
            self.currentProject.TransactionManager.AddEndMark(Client.CommandGranularity.Command)

        if (self.lineCtl.AutoTabExtSkip):
            Keyboard.Focus(self.coordCtl)
        else:
            Keyboard.Focus(self.lineCtl)
        return